﻿################################
## Farm Initial Configuration ## 
################################

Add-PSSnapin Microsoft.SharePoint.PowerShell -erroraction SilentlyContinue 

## Settings ##
$databaseServerName = "SP2013SQL" 
$saAppPoolName = "SharePoint Web Services"
$appPoolUserName = "SP2013\SPService"

## Service Application Service Names ## 
$usageSAName = "Usage and Health Data Collection Service"
$stateSAName = "State Service"
$metadataSAName = "Managed Metadata Service"
$stateServiceDatabaseName = "SP2013_Farm_StateDB"
$appManagementName = "App Management Service"
$appManagementDatabaseName = "SP2013_Farm_AppManagementDB"
$subSettingstName = "Subscription Settings Service"
$subSettingstDatabaseName = "SP2013_Farm_SubSettingsDB"

# Retrieve or create the services application pool and managed account
$saAppPool = Get-SPServiceApplicationPool -Identity $saAppPoolName -EA 0 
if($saAppPool -eq $null) 
{ 
  Write-Host "Creating Service Application Pool..."
  
  $appPoolAccount = Get-SPManagedAccount -Identity $appPoolUserName -EA 0 
  if($appPoolAccount -eq $null) 
  { 
      Write-Host "Please supply the password for the Service Account..."
      $appPoolCred = Get-Credential $appPoolUserName
      $appPoolAccount = New-SPManagedAccount -Credential $appPoolCred -EA 0 
  } 
  
  $appPoolAccount = Get-SPManagedAccount -Identity $appPoolUserName -EA 0 
  
  if($appPoolAccount -eq $null) 
  { 
    Write-Host "Cannot create or find the managed account $appPoolUserName, please ensure the account exists."
    Exit -1 
  } 
  
  New-SPServiceApplicationPool -Name $saAppPoolName -Account $appPoolAccount -EA 0 > $null
      
} 

# Configure the web analytics and health data collection service before creating the service application
Set-SPUsageService -LoggingEnabled 1 -UsageLogLocation "C:\Program Files\Common Files\Microsoft Shared\Web Server Extensions\15\LOGS\" -UsageLogMaxSpaceGB 5

# Usage Service
Write-Host "Creating Usage Service and Proxy..."
$serviceInstance = Get-SPUsageService
New-SPUsageApplication -Name $usageSAName -DatabaseServer $databaseServerName -DatabaseName "SP2013_Farm_UsageDB" -UsageService $serviceInstance > $null

# State Service
$stateServiceDatabase = New-SPStateServiceDatabase -Name $stateServiceDatabaseName
$stateSA = New-SPStateServiceApplication -Name $stateSAName -Database $stateServiceDatabase
# $stateSA = Get-SPServiceApplication | where-object {$_.DisplayName -eq "State Service"}
New-SPStateServiceApplicationProxy -ServiceApplication $stateSA -Name "$stateSAName Proxy" -DefaultProxyGroup

# Managed Metadata Service
Write-Host "Creating Metadata Service and Proxy..."
New-SPMetadataServiceApplication -Name $metadataSAName -ApplicationPool $saAppPoolName -DatabaseServer $databaseServerName -DatabaseName "SP2013_Farm_MetadataDB" > $null
New-SPMetadataServiceApplicationProxy -Name "$metadataSAName Proxy" -DefaultProxyGroup -ServiceApplication $metadataSAName > $null
Get-SPServiceInstance | where-object {$_.TypeName -eq "Managed Metadata Web Service"} | Start-SPServiceInstance > $null

###################################
## Scalable Search Configuration ## 
###################################

##########################
# Search Service - START #
##########################
# http://technet.microsoft.com/en-us/library/jj219705(v=office.15)
# http://technet.microsoft.com/en-us/library/ee792877(v=office.15).aspx

$searchMachines = @("SP2013APP1", "SP2013APP2")
$searchAdminComponentMachines = @("SP2013APP1", "SP2013APP2")
$searchAnalyticsMachines = @("SP2013APP1", "SP2013APP2")
$searchContentProcessingMachines = @("SP2013APP1", "SP2013APP2")
$searchQueryMachines = @("SP2013APP1", "SP2013APP2")
$searchCrawlerMachines = @("SP2013APP1", "SP2013APP2")
$searchIndexMachines = @("SP2013APP1", "SP2013APP2")

$searchSAName = "Search Service"
$saAppPoolName = "SharePoint Web Services"
$databaseServerName = "SP2013SQL"
$searchDatabaseName = "SP2013_Farm_Search"
$indexLocation = "C:\SearchIndex"

Write-Host "Creating Search Service and Proxy..."
Write-Host "  Starting Services..."

foreach ($machine in $searchMachines)
{
    Write-Host "    Starting Search Services on $machine"
    Start-SPEnterpriseSearchQueryAndSiteSettingsServiceInstance $machine -ErrorAction SilentlyContinue
    Start-SPEnterpriseSearchServiceInstance $machine -ErrorAction SilentlyContinue
}

Write-Host "  Creating Search Application..."
$searchApp = Get-SPEnterpriseSearchServiceApplication -Identity $searchSAName -ErrorAction SilentlyContinue
if (!$searchApp)
{
    $searchApp = New-SPEnterpriseSearchServiceApplication -Name $SearchSAName -ApplicationPool $saAppPoolName -DatabaseServer $databaseServerName -DatabaseName $searchDatabaseName
}
$searchInstance = Get-SPEnterpriseSearchServiceInstance -Local

# Define the search topology
Write-Host "  Defining the Search Topology..."
$initialSearchTopology = $searchApp | Get-SPEnterpriseSearchTopology -Active 
$newSearchTopology = $searchApp | New-SPEnterpriseSearchTopology 

# Create search components
Write-Host "  Creating Admin Component..."
foreach ($machine in $searchAdminComponentMachines)
{
    New-SPEnterpriseSearchAdminComponent -SearchTopology $newSearchTopology -SearchServiceInstance $machine
}

Write-Host "  Creating Analytics Component..."
foreach ($machine in $searchAnalyticsMachines)
{
    New-SPEnterpriseSearchAnalyticsProcessingComponent -SearchTopology $newSearchTopology -SearchServiceInstance $machine
}

Write-Host "  Creating Content Processing Component..."
foreach ($machine in $searchContentProcessingMachines)
{
    New-SPEnterpriseSearchContentProcessingComponent -SearchTopology $newSearchTopology -SearchServiceInstance $machine
}

Write-Host "  Creating Query Processing Component..."
foreach ($machine in $searchQueryMachines)
{
    New-SPEnterpriseSearchQueryProcessingComponent -SearchTopology $newSearchTopology -SearchServiceInstance $machine
}

Write-Host "  Creating Crawl Component..."
foreach ($machine in $searchCrawlerMachines)
{
    New-SPEnterpriseSearchCrawlComponent -SearchTopology $newSearchTopology -SearchServiceInstance $machine 
}

Write-Host "  Creating Index Component..."
foreach ($machine in $searchIndexMachines)
{
    New-SPEnterpriseSearchIndexComponent -SearchTopology $newSearchTopology -SearchServiceInstance $machine -RootDirectory $indexLocation 
}

Write-Host "  Activating the new topology..."
$newSearchTopology.Activate()

Write-Host "  Creating Search Application Proxy..."
$searchProxy = Get-SPEnterpriseSearchServiceApplicationProxy -Identity "$searchSAName Proxy" -ErrorAction SilentlyContinue
if (!$searchProxy)
{
    New-SPEnterpriseSearchServiceApplicationProxy -Name "$searchSAName Proxy" -SearchApplication $searchSAName
}

########################
# Search Service - END #
########################

# User Profile Service
Write-Host "Creating User Profile Service and Proxy acting as Farm account..."
$sb = {
	Add-PSSnapin Microsoft.SharePoint.PowerShell -erroraction SilentlyContinue 

	Write-Host "Creating User Profile Service and Proxy acting as Farm account..."
	$saAppPoolNameForUPS = "SharePoint Web Services"
	$saAppPoolForUPS = Get-SPServiceApplicationPool -Identity $saAppPoolNameForUPS -EA 0 
	$userUPSName = "User Profile Service"	
	$databaseServerNameForUPS = "SP2013SQL"

	$userProfileService = New-SPProfileServiceApplication -Name $userUPSName -ApplicationPool $saAppPoolNameForUPS -ProfileDBServer $databaseServerNameForUPS -ProfileDBName "SP2013_Farm_ProfileDB" -SocialDBServer $databaseServerNameForUPS -SocialDBName "SP2013_Farm_SocialDB" -ProfileSyncDBServer $databaseServerNameForUPS -ProfileSyncDBName "SP2013_Farm_SyncDB"

	New-SPProfileServiceApplicationProxy -Name "$userUPSName Proxy" -ServiceApplication $userProfileService -DefaultProxyGroup > $null
}

$farmAccount = (Get-SPFarm).DefaultServiceAccount
$farmCredential = Get-Credential $farmAccount.Name
$job = Start-Job -Credential $farmCredential -ScriptBlock $sb | Wait-Job

Get-SPServiceInstance | where-object {$_.TypeName -eq "User Profile Service"} | Start-SPServiceInstance > $null

$upa = Get-SPServiceApplication -Name "User Profile Service" # –Identity f0952704-5b8d-40bd-8f5a-f3b44ff6a419
$upa.NetBiosDomainNamesEnabled = 1
$upa.Update()

# After configuring security, execute the following line
Restart-Service sptimerv4

######################################
# Configure Apps Environment - START #
######################################

Write-Host "Creating Subscription Settings Service and Proxy..."
$subSvc = New-SPSubscriptionSettingsServiceApplication –ApplicationPool $saAppPoolName –Name $subSettingstName –DatabaseName $subSettingstDatabaseName
$subSvcProxy = New-SPSubscriptionSettingsServiceApplicationProxy –ServiceApplication $subSvc
Get-SPServiceInstance | where-object {$_.TypeName -eq $subSettingstName} | Start-SPServiceInstance > $null

Write-Host "Creating App Management Service and Proxy..."
$appManagement = New-SPAppManagementServiceApplication -Name $appManagementName -DatabaseServer $databaseServerName -DatabaseName $appManagementDatabaseName –ApplicationPool $saAppPoolName
$appManagementProxy = New-SPAppManagementServiceApplicationProxy -ServiceApplication $appManagement -Name "$appManagementName Proxy"
Get-SPServiceInstance | where-object {$_.TypeName -eq $appManagementName} | Start-SPServiceInstance > $null

$AppsSiteRootOwner = "SP2013\Administrator"
$AppsSiteRootName = "Apps Portal"
$AppsSiteRootTemplate = Get-SPWebTemplate "STS#1"  
New-SPSite -Url "http://sp2013wfe1/" -OwnerAlias $AppsSiteRootOwner -Template $AppsSiteRootTemplate -Name $AppsSiteRootName

Set-SPAppDomain sp2013apps.test
Set-SPAppSiteSubscriptionName -Name "apps" -Confirm:$false

####################################
# Configure Apps Environment - END #
####################################

## Configure Service Applications Groups ##
New-SPServiceApplicationProxyGroup -Name "IntranetProxyGroup"

Get-SPServiceApplicationProxy
Get-SPServiceApplicationProxyGroup

Add-SPServiceApplicationProxyGroupMember "IntranetProxyGroup" –Member c1bf65a5-face-4e28-bdc4-b08443caf619 # State Service Proxy
Add-SPServiceApplicationProxyGroupMember "IntranetProxyGroup" –Member 2bc76c64-7fbc-4304-820c-ffa481864b30 # MMS Proxy
Add-SPServiceApplicationProxyGroupMember "IntranetProxyGroup" –Member a8ff2d9e-9ff3-4897-a9dc-8276d09a3eae # Search Proxy
Add-SPServiceApplicationProxyGroupMember "IntranetProxyGroup" –Member fdbe4725-857c-4f25-9178-ad8e1952e58b # Usage and Health Proxy
Add-SPServiceApplicationProxyGroupMember "IntranetProxyGroup" –Member 443afbf8-15a4-4094-91a0-20fd6a3dfb0b # App Management Proxy
Add-SPServiceApplicationProxyGroupMember "IntranetProxyGroup" –Member ec4277aa-1c29-401e-89fb-6e9959d80615 # Subscription Settings Proxy
Add-SPServiceApplicationProxyGroupMember "IntranetProxyGroup" –Member 1af96eb1-576c-4f4a-91d4-c6dc262e29db # UPS


